<?php

/*
Copyright 2013 Bing Technologies Pty Lid. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE BING TECHNOLOGIES PTY.LTD.  "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL BING TECHNOLOGIES PTY. LTD. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are
those of the authors and should not be interpreted as representing official
policies, either expressed or implied, of Bing Technologies Pty. Ltd.
*/

//debug mode
$DEBUG = false;

if (!$DEBUG) {
	//in the debug mode this is not needed
	ob_start();
}

//load the library
require(__DIR__ . '/easypost_api.php');

//create the uploader object
$obj = new ep_uploader();

//load a config file
$obj->load_config_ini(__DIR__ . '/ep_api.ini.php');

//how to deal with errors (general php stuff)
error_reporting(E_ALL);

ini_set('display_errors', True);
error_reporting(E_ALL);

set_error_handler('exceptions_error_handler');

function exceptions_error_handler($severity, $message, $filename, $lineno) {
  if (error_reporting() == 0) {
    return;
  }
  if (error_reporting() & $severity) {
    throw new ErrorException($message, 0, $severity, $filename, $lineno);
  }
}

//http redirect depending on the DEBUG mode
function redirect($url)
{
	global $DEBUG;
	if ($DEBUG) {
		echo '<p><a href="'.$url.'">Continue</a></p>';
	} else {
		header('Location: '.$url);
	}
}

if ($DEBUG) {
	//setting debug mode
	$obj->allow_debug();
	$obj->set_debug_echo_line_separator('<br/>');
}


if (empty($_POST)) {
	//if POST data is empty then show the form
	//echoing html stuff including javascript code
	echo '
	<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
	<html>
	<head>
	<meta http-equiv="X-UA-Compatible" content="IE=100" > <!-- IE9 mode -->
	<!-- CSS goes in the document HEAD or added to your external stylesheet -->
	<style type="text/css">
		table.gridtable {
			font-family: verdana,arial,sans-serif;
			font-size:11px;
			color:#333333;
			border-width: 1px;
			border-color: #666666;
			border-collapse: collapse;
		}
		table.gridtable th {
			border-width: 1px;
			padding: 8px;
			border-style: solid;
			border-color: #666666;
			background-color: #dedede;
		}
		table.gridtable tr {
			background-color: #d4e3e5;
		}
		table.gridtable td {
			border-width: 1px;
			padding: 8px;
			border-style: solid;
			border-color: #666666;
		}
	</style>
	<script type="text/javascript">
		//remove empty values from an array
		function removeEmpty(arr) {
			var rs = new Array();
			for(var i=0; i<arr.length; i++) {
				if(arr[i]) {
					rs.push(arr[i]);
				}
			}
			return rs;
		}
		//submitting the form
		function sendFile(form)
		{
			var postals = new Array();
			for(var i=0; i<addrdata.length; i++) {
				var p = addrdata[i];
				if ( p.constructor == Postal ) {
					postals.push({ "postal": removeEmpty(p.postal) });
				} else {
					postals.push({ "email": p.email });
				}
			}
			form.json_data.value = JSON.stringify(postals);
			form.submit();
			return false;
		}
		//update the address table view using stored array of addresses
		function updateAddressTable()
		{
			var submitform = document.getElementById("SendForm");
			var tbody = document.getElementById("vtable").getElementsByTagName("tbody")[0];
			while (tbody.firstChild) {
				tbody.removeChild(tbody.firstChild);
			}
			submitform.sendit.disabled = true;
			for(var i=0; i<addrdata.length; i++) {
				var tr = document.createElement("tr");
				tbody.appendChild(tr);
				tr.onmouseover = (function() {
					var etr = tr;
					return function() {
						etr.style.backgroundColor="#e4f3f5"; 
					};
				})();
				tr.onmouseout = (function() {
					var etr = tr;
					return function() {
						etr.style.backgroundColor="#d4e3e5"; 
					};
				})();
				var addr = addrdata[i];
				if (addr.constructor == Postal) {
					for (var j=0; j<addr.postal.length; j++) {
						var td = document.createElement("td");
						td.appendChild(document.createTextNode(addr.postal[j]));
						tr.appendChild(td);
					}
				} else {
					var td = document.createElement("td");
					td.setAttribute("colspan", 4);
					td.appendChild(document.createTextNode(addr.email));
					tr.appendChild(td);
				}
				var td = document.createElement("td");
				var b = document.createElement("button");
				b.onclick = (function() {
					var currentI = i;
					return function() { 
						deleteAddress(currentI);
					};
				})();
				b.appendChild(document.createTextNode("Delete"));
				td.appendChild(b);
				tr.appendChild(td);
				submitform.sendit.disabled = false;
			}
		}
		//delete and address from the address array by its index
		function deleteAddress(i)
		{
			addrdata.splice(i, 1);
			updateAddressTable();
		}
		//create a postal address object for the address array
		function Postal(form) {
			this.postal = new Array(form.a_line1.value, form.a_line2.value, form.a_line3.value, form.a_line4.value);
		}
		//create an email address object for the address array
		function Email(form) {
			this.email = form.a_email.value;
		}
		//add a postal address to the address array
		function addAddress(form) 
		{
			addrdata.push(new Postal(form));
			updateAddressTable();
			form.a_line1.value = "";
			form.a_line2.value = "";
			form.a_line3.value = "";
			form.a_line4.value = "";
			form.a_line1.focus();
			return false;
		}
		//add an email address to the address array
		function addEmail(form) 
		{
			addrdata.push(new Email(form));
			updateAddressTable();
			form.a_email.value = "";
			form.a_email.focus();
			return false;
		}
		//init the global address array
		function onLoad()
		{
			addrdata = new Array();
		}
	</script>
	</head>
	<body  onLoad="onLoad();">
	';
	//echoing message about the previous upload if any
	if (array_key_exists('filename', $_GET) && array_key_exists('postal', $_GET)) {
		echo 'ID: '.$_GET['session_id'].'</p>';
		if(isset($_GET['errs'])) {
			echo '<p>';
			foreach($_GET['errs'] as $err) {
				echo htmlspecialchars($err).'<br/>';
			}
		} else {
			echo '<p>File "'.htmlspecialchars($_GET['filename']).'" has been uploaded for delivery to: <br/>'.implode(array_map('htmlspecialchars', explode(';', $_GET['postal'])), '<br/>').'<br/>';
		}
	}
	//echoing error message from the previous upload if any
	if (array_key_exists('error', $_GET)) {
		echo '<p><font color="red">Error while uploading: '.htmlspecialchars($_GET['error']).'</font></p>';
	}
	//store url for the form action attribute
	$current_url = explode("?", $_SERVER['REQUEST_URI']);
	//the add address form
	echo '
	<form action="" onsubmit="return false;">
	<p>
	Postal Address<br/>
	<input type="input" name="a_line1"/><br/>
	<input type="input" name="a_line2"/><br/>
	<input type="input" name="a_line3"/><br/>
	<input type="input" name="a_line4"/><br/>
	<input type="submit" name="submitAddress" value="Add address" onclick="addAddress(this.form)"/><br/>
	</p>
	<p>
	E-mail<br/>
	<input type="input" name="a_email"/><br/>
	<input type="submit" name="submitEmail" value="Add e-mail" onclick="addEmail(this.form)"/><br/>
	</p>
	</form>
	<table id="vtable" class="gridtable">
	<thead>
	<tr>
	<th colspan="4">Receipients</th>
	</tr>
	</thead>
	<tbody>
	</tbody>
	</table>';
	//the upload forms
	echo '<form action="'.$current_url[0].'" method="post" enctype="multipart/form-data" onsubmit="return sendFile(this);" id="SendForm">
	<input type="hidden" name="json_data"/>
	<table>
		<tr>
			<td>Orientation</td>
			<td>
				<select name="orientation">
					<option value="portrait">Portrait</option>
					<option value="landscape">Landscape</option>
				</select>
			</td>
		</tr>
		<tr>
			<td>Colour model</td>
			<td>
				<select name="colour_model">
					<option value="colour">Colour</option>
					<option value="mono">Mono</option>
				</select>
			</td>
		</tr>
		<tr>
			<td>Duplex</td>
			<td><input type="checkbox" name="duplex" value="1" checked></td>
		</tr>
		<tr>
			<td>PDF file:</td><td><input type="file" name="filename" id="file"/></td>
		</tr>
		<tr>
			<td colspan="2"><input type="submit" name="sendit" value="Submit" disabled/></td>
		</tr>
	</table>
	</form>
	</body>
	</html>
	';
} else {
	//POST data submitted
	$filename = $_FILES["filename"]["name"];
	if (empty($filename)) {
		//if no file is submitted return an error to the browser
		header('Location: '.$_SERVER["REQUEST_URI"].'?error='.urlencode('No file provided'));
	} else {
		//json_data contains postal and email addresses like [{'email': EMAIL}, {'postal': [POSTAL LINE1, POSTAL LINE2, ...]}, ...]
		$addresses = json_decode($_POST['json_data'], TRUE);
		try {
			//create an array that is going to become eventually a json array
			$letters = array(
				"duplex" => (array_key_exists('duplex', $_POST)) ? true : false,
				"colour_model" => $_POST['colour_model'],
				"orientation" => $_POST['orientation'],
				'letters' => $addresses
			);
			//create a temporary json file for the upload
			$jsontmpfname = $obj->make_json_file($letters);
			//output the contents of the file in debug mode
			$obj->echo_log("\nJson:\n".var_export(file_get_contents($jsontmpfname), TRUE));
			//make the json file destination name
			$jsonfname = explode('.', $_FILES["filename"]['name']);
			//the same as the uploading pdf name but with json extension
			$jsonfname = $jsonfname[0].'.json';
			//call the upload object method
			$reply = $obj->do_upload(array(array($_FILES["filename"]["tmp_name"], $_FILES["filename"]['name']), array($jsontmpfname, $jsonfname)));
			//the file have been uploaded by now
			//so the temporary json file is to be removed
			unlink($jsontmpfname);
			//make a status message for the browser
			$sent_to = array();
			foreach($addresses as $p) {
				array_push($sent_to, (in_array('postal', array_keys($p))) ? 'Postal: '.implode(', ', $p['postal']) : 'Email: '.$p['email']);
			}
			//redirect the browser to the url without POST data to avoid repeated submissions sending status messages as a GET parameters
			$rurl = $_SERVER["REQUEST_URI"].'?filename='.urlencode($filename).'&postal='.urlencode(implode(';', $sent_to)).'&session_id='.urlencode($reply['session_id']);
			if(isset($reply['errs'])) {
				foreach($reply['errs'] as $err) {
					$rurl .= '&errs[]='.urlencode($err);
				}
			}
			redirect($rurl);
		} catch(Exception $e) {
			//in case of an exception create an error status message
			if ( $e->getCode() == 401 ) {
				$emsg = 'Authentication error';
			} else {
				$emsg = $e->getMessage();
			}
			//echo the error message in case of the debug mode
			$obj->echo_log($e->getMessage()."\n");
			$obj->echo_log($e->getTraceAsString()."\n");
			//if the error message is 401 http error just tell that username and password are invalid
			if(preg_match('/^HTTP\/1.[0-9] 401/', $emsg)) {
				$emsg = 'The account username or password is incorrect';
			}
			//and redirect the browser to the url without POST data with the error message as a GET parameter
			redirect($_SERVER["REQUEST_URI"].'?error='.urlencode($emsg));
		}
	}
}

?>
